/*
Data: 2024-10-10
SOFTWARE DIMOSTRATIVO per l'interfacciamento a ai4call.

IL SOFTWARE VIENE FORNITO "COSÌ COM'È", SENZA GARANZIE DI ALCUN TIPO, ESPRESSE O IMPLICITE, INCLUSE, MA NON SOLO, LE GARANZIE DI COMMERCIABILITÀ, 
IN NESSUN CASO GLI AUTORI O I DETENTORI DEI DIRITTI D'AUTORE SARANNO RESPONSABILI DI EVENTUALI RECLAMI, DANNI O ALTRE RESPONSABILITÀ, SIA IN 
UN'AZIONE LEGALE CHE CONTRATTUALE, EXTRACONTRATTUALE O DI ALTRO TIPO, DERIVANTE DALL'UTILIZZO, ANCHE PARZIALE, DEL SOFTWARE.
*/

const express = require("express");
const bodyParser = require("body-parser");
const cors = require("cors");
const fs = require("fs");
const path = require("path");
const fetch = require("node-fetch");
const nodemailer = require("nodemailer");
const { Lame } = require("node-lame");
const app = express();
const port = 3210;

// Configurazione di base per Express: abilitazione del supporto CORS e Body Parser per gestire i dati JSON di grandi dimensioni
app.use(cors());
app.use(bodyParser.json({ limit: "10gb" })); // Imposta un limite massimo per i payload JSON
app.use(bodyParser.urlencoded({ extended: false }));

// Log iniziale per indicare l'avvio del server
console.log(" ");
console.log(
  "-------------------------------------------------------------------------------------"
);
console.log(" ");
app.listen(port, () =>
  console.log(
    "progetto-ai4call (demo-segreteria) in ascolto sulla porta: ",
    port,
    "\n"
  )
);

//---------------------------------------------------------------------------------------
// PRIMA REST DOVE ENTRERÀ LA CHIAMATA
// Questa è la prima rotta POST a cui si invia la chiamata iniziale.
// Questa è la FIRST URL che deve essere dichiarata sul progetto dal portale ai4call.com
// i numeri di telefono del chiamante e del destinatario, e si decide l'azione successiva
// in base al tipo di numero (mobile, fisso, estero, anonimo).
//---------------------------------------------------------------------------------------

let callControl = {};
app.post("/prima-richiesta", async (req, res) => {
  try {
    let Call_Id = req.headers["x-call-id"]; // Identificativo della chiamata passato nell'header
    console.log(
      "\n---------------------------------------------------------------------- "
    );
    console.log(
      "--------------------------START DEMO AI4CALL ----------------------------------- "
    );

    console.log("\nai4call ha chiamato mia API: 'prima-richiesta' con: \n");
    console.log("-- HEADER:", JSON.stringify(req.headers)); // Stampa gli header
    console.log("-- BODY:", JSON.stringify(req.body)); // Stampa il corpo della richiesta

    // Estrai i numeri di telefono del chiamante e del destinatario dal body della richiesta
    let { recipientNumber, callerNumber } = req.body;
    let sendresponse;

    // Chiamata API con gestione errori
    // Richiesta HTTP a una API REST esterna per ottenere messaggi o dati relativi al destinatario

    let datiAPI;
    try {
      datiAPI = await chiamaRestAPI(recipientNumber);
    } catch (apiError) {
      console.error("Errore durante la chiamata all'API:", apiError);
      sendresponse = {
        text: "IL numero non è attivo, arrivederci",
        action: "hangup",
      };
      return res.json(sendresponse);
    }

    // Se l'API non restituisce dati validi
    if (!datiAPI) {
      sendresponse = {
        text: "IL numero non è attivo, arrivederci",
        action: "hangup",
      };
      return res.json(sendresponse);
    }

    // Destrutturazione sicura con valori predefiniti per evitare campi mancanti
    const {
      messaggi: { messaggio_segreteria = "", messaggio_default = "" } = {},
      conf_email: { to = "", oggetto_email = "", body = "" } = {},
      configurazione_accesso_email: {
        host_email = "",
        port_email = null,
        user_email = "",
        pass_email = "",
        from = "",
      } = {},
    } = datiAPI;

    // Inizializza i dati della chiamata
    callControl[Call_Id] = {
      callerNumber,
      recipientNumber,
      messaggio_segreteria,
      messaggio_default,
      to,
      oggetto_email,
      body,
      host_email,
      port_email,
      user_email,
      pass_email,
      from,
    };

    // Controlla il tipo di numero chiamante
    // Verifica il tipo di numero (mobile, fisso, estero, anonimo)
    let tipoNumero;
    try {
      tipoNumero = await checkNumberType(callerNumber);
    } catch (numberError) {
      console.error(
        "Errore durante il controllo del tipo di numero:",
        numberError
      );
    }

    //------------------------------------------------------------------------------------------
    // Logica di gestione del tipo di numero
    // Se il numero chiamante è un numero MOBILE inviamo un messaggio whatsapp
    // Se il numero chiamante è un numero ANONIMO/FISSO diamo messaggio audio
    // Se il numero chiamante è un numero ESTERO chiudiamo la chiamata
    //------------------------------------------------------------------------------------------

    switch (tipoNumero) {
      case "estero":
        // [ RESPONSE TO AI4CALL - ACTION HANGUP - chiude la chiamata]
        sendresponse = {
          text: "",
          action: "hangup",
        };
        break;

      default:
        // [ RESPONSE TO AI4CALL - ACTION VOICEMAIL - // Il flusso proseguirà verso la next_url indicata, dove verrà gestito l'invio dell'email con il messaggio trascritto e il file audio in allegato . ]
        sendresponse = {
          text: callControl[Call_Id].messaggio_segreteria,
          action: "voicemail",
          next_url:
            "http://[ip o dominio del server]:[porta dove è in ascolto il server]/ricevo-segreteria",
          key: "messaggio_segreteria",
          transcription_key: "messaggio_in_testo",
        };
        break;
    }
    // INVIA LA RISPOSTA AD AI4CALL
    console.log(
      "\n-- =====> Io gli mando questo response:",
      JSON.stringify(sendresponse)
    );
    res.json(sendresponse);
  } catch (error) {
    console.error("Errore durante il processo generale:", error);
    sendresponse = {
      text: "IL numero non è attivo, arrivederci",
      action: "hangup",
    };
    res.json(sendresponse);
  }
});

//---------------------------------------------------------------------------------------
// SECONDA REST PER LA GESTIONE DELLA SEGRETERIA
// Qui viene gestito il messaggio della segreteria.
// Viene creato un file audio in formato WAV e MP3, e poi inviato via email al destinatario
// insieme alla trascrizione del messaggio.
//---------------------------------------------------------------------------------------

app.post("/ricevo-segreteria", async (req, res) => {
  let Call_Id = req.headers["x-call-id"];
  console.log(
    "\n---------------------------------------------------------------------- \n"
  );
  console.log("\n-- ai4call ha chiamato mia API: 'ricevo-segreteria' \n");

  // Estrazione dei dati dal body della richiesta: il messaggio della segreteria e la trascrizione
  let messaggioSegreteria = req.body.messaggio_segreteria;
  let messaggio_in_testo = req.body.messaggio_in_testo;

  // Definizione dei percorsi dove verranno salvati i file audio (WAV e MP3)
  const audioFilePathWav = path.join(
    __dirname,
    `/audio/segreteria_${callControl[Call_Id].callerNumber}.wav`
  );
  const audioFilePathMp3 = path.join(
    __dirname,
    `/audio/segreteria_${callControl[Call_Id].callerNumber}.mp3`
  );

  // Decodifica del messaggio vocale ricevuto in base64 in formato WAV e salvataggio del file
  const bufferWav = Buffer.from(messaggioSegreteria, "base64");

  try {
    // Salvataggio del file audio WAV
    await fs.promises.writeFile(audioFilePathWav, bufferWav);
    // Conversione del file WAV in MP3
    const encoder = new Lame({
      output: audioFilePathMp3,
      bitrate: 128,
    }).setFile(audioFilePathWav);
    await encoder.encode();
    console.log("File audio .mp3 salvato con successo:", audioFilePathMp3);
    console.log("File audio .wav salvato con successo:", audioFilePathWav);
    console.log("Messaggio trascritto:", messaggio_in_testo);

    // Lettura dei file WAV e MP3 per l'invio tramite email
    const fileWav = await fs.promises.readFile(audioFilePathWav);
    const fileMp3 = await fs.promises.readFile(audioFilePathMp3);

    // Invia l'email con i file audio e la trascrizione del messaggio
    await sendMyEmail(Call_Id, fileWav, fileMp3, messaggio_in_testo);

    console.log(
      "\n----------- Processo completato con successo! ------------------------ \n"
    );
  } catch (emailError) {
    // Gestione degli errori in caso di problemi durante l'invio dell'email
    console.error("Errore durante l'invio dell'email:", emailError);
    sendresponse = {
      text: "IL numero non è attivo, arrivederci",
      action: "hangup",
    };
    return res.json(sendresponse);
  }
});

app.post("/hangup", async (req, res) => {
  let Call_Id = req.headers["x-call-id"];
  console.log(
    "\n---------------------------------------------------------------------- \n"
  );
  console.log("ai4call ha chiamato mia API: 'hangup' con:");
  console.log("-- BODY:", JSON.stringify(req.body));
  if (callControl[Call_Id]) {
    delete callControl[Call_Id];
    console.log(`Processo completato per Call_Id ${Call_Id}.`);
  } else {
    console.log(`Nessun dato trovato per Call_Id ${Call_Id}.`);
  }
  console.log(
    "\n----------- Processo completato con successo! ------------------------ \n"
  );
});

//---------------------------------------------------------------------------------------
// Funzione per verificare il tipo di numero (mobile, fisso, estero, anonimo)
// Questa funzione analizza il numero di telefono per determinare il tipo e gestirlo
// di conseguenza.
//---------------------------------------------------------------------------------------
async function checkNumberType(number) {
  try {
    if (number.startsWith("+39")) {
      number = number.slice(3); // Rimuovi il prefisso internazionale per numeri italiani
    }

    if (number.startsWith("3")) {
      return "mobile_italiano"; // Numeri mobili iniziano con 3 in Italia
    } else if (number.startsWith("0")) {
      return "fisso_italiano"; // Numeri fissi iniziano con 0
    } else if (number.startsWith("+")) {
      return "estero"; // Numeri esteri hanno il prefisso +
    } else if (number === "anonymous") {
      return "anonymous"; // Gestione numeri anonimi
    } else {
      return "numero_sconosciuto"; // Numeri non riconosciuti
    }
  } catch (err) {
    console.error("Errore nel check del numero:", err);
  }
}
//---------------------------------------------------------------------------------------
// Funzione per inviare un'email con il messaggio della segreteria
// Questa funzione invia un'email con i file audio in allegato (WAV e MP3) e la trascrizione
// del messaggio della segreteria.
// Utilizza i dati modificati in rest/config.json, inclusi i parametri di configurazione
// per l'accesso all'email (host, porta, utente, password) e il contenuto personalizzato
// dell'email (destinatario, oggetto, corpo).
//---------------------------------------------------------------------------------------

async function sendMyEmail(Call_Id, fileWav, fileMp3, trascrizione) {
  try {
    const mail = nodemailer.createTransport({
      host: callControl[Call_Id].host_email,
      port: callControl[Call_Id].port_email,
      auth: {
        user: callControl[Call_Id].user_email,
        pass: callControl[Call_Id].pass_email,
      },
    });

    await mail.sendMail({
      from: callControl[Call_Id].from,
      to: callControl[Call_Id].to,
      subject: `${callControl[Call_Id].oggetto_email} ${callControl[Call_Id].callerNumber}`,
      html: `${callControl[Call_Id].body} ${trascrizione}`,
      attachments: [
        {
          filename: `messaggio_segreteria_${callControl[Call_Id].callerNumber}.wav`,
          content: fileWav,
          contentType: "audio/wav",
        },
        {
          filename: `messaggio_segreteria_${callControl[Call_Id].callerNumber}.mp3`,
          content: fileMp3,
          contentType: "audio/mpeg",
        },
      ],
    });
    console.log("Email inviata con successo!");
  } catch (error) {
    console.log("Errore nell'invio dell'email", error);
  }
}
//---------------------------------------------------------------------------------------
// Funzione per chiamare l'API REST
// Questa funzione effettua una richiesta HTTP a una API REST esterna per ottenere
// messaggi o dati relativi al destinatario. Restituisce il JSON ottenuto dalla richiesta.
//---------------------------------------------------------------------------------------
async function chiamaRestAPI(recipientNumber) {
  const url = `http://localhost:[porta server rest/esempio-rest-dati.js]/dati/${recipientNumber}`;
  const response = await fetch(url);
  if (!response.ok) {
    throw new Error(`Errore nella richiesta: ${response.status}`);
  }

  const data = await response.json(); // Estrai i dati dal corpo della risposta
  return data;
}
